@@ -1,27 +1,21 @@
-"""
-Derived module from spodbase.py for classic spod.
-"""
+"""Derived module from spodbase.py for classic spod."""
 
 # import standard python packages
 import os
-import sys
 import numpy as np
 from scipy.io import loadmat
 import matplotlib as mpl
 import matplotlib.pyplot as plt
 from matplotlib import animation
-import matplotlib.gridspec as gridspec
-from matplotlib import colors as mcolors
 from mpl_toolkits.axes_grid1 import make_axes_locatable
 mpl.rc('figure', max_open_warning = 0)
 from os.path import splitext
-import warnings
 
 # Current, parent and file paths
 CWD = os.getcwd()
 CF = os.path.realpath(__file__)
 CFD = os.path.dirname(CF)
-BYTE_TO_GB = 9.3132257461548e-10
+
 
 
 # getters
@@ -48,7 +42,7 @@ def find_nearest_coords(coords, x, data_space_dim):
 	Get nearest data coordinates to requested coordinates `coords`.
 
 	:param np.ndarray coords: coordinate requested.
-	:param np.ndarray x: data coordinates.
+	:param list x: data coordinates.
 	:param int: spatial dimension of the data.
 
 	:return: the nearest coordinate to the `coords` requested and its id.
@@ -56,15 +50,17 @@ def find_nearest_coords(coords, x, data_space_dim):
 	"""
 	coords = np.asarray(coords)
 	if isinstance(x, list):
-		grid = np.array(np.meshgrid(*x))
-	elif isinstance(x,np.ndarray) == data:
-		if x.shape == data_space_dim:
-			grid = x
+		grid = np.array(np.meshgrid(*x, indexing='ij'))
 	else:
+		raise TypeError('`x` must be a list.')
+
+	# check dimensions
+	if grid[0,::].shape != data_space_dim:
 		raise ValueError('Dimensions of coordinates `x` does not match data.')
+
 	idx = tuple()
 	xi  = tuple()
-	for i,coord in enumerate(coords):
+	for i, coord in enumerate(coords):
 		cnt = len(grid[i,::].shape) - i - 1
 		tmp = np.abs(grid[i,::] - coord)
 		tmp_idx = np.unravel_index(np.argmin(tmp), tmp.shape)
@@ -110,13 +106,13 @@ def get_mode_from_file(filename):
 		file at a given frequency.
 	:rtype: numpy.ndarray
 	"""
-	basename, ext = splitext(filename)
+	_, ext = splitext(filename)
 	if ext.lower() == '.npy':
 		m = np.load(filename)
-	elif ext.lower() == '.mat':
-		pass
-	elif ext.lower() == 'nc':
-		pass
+	# elif ext.lower() == '.mat':
+	# 	pass
+	# elif ext.lower() == 'nc':
+	# 	pass
 	else:
 		raise ValueError(ext, 'file extension not recognized.')
 	return m
@@ -181,12 +177,7 @@ def plot_eigs(eigs, title='', figsize=(12,8), show_axes=True,
 		ax.set_aspect('equal')
 
 	# save or show plots
-	if filename:
-		if path == 'CWD': path = CWD
-		plt.savefig(os.path.join(path,filename), dpi=200)
-		plt.close()
-	else:
-		plt.show()
+	_save_show_plots(filename, path, plt)
 
 
 
@@ -214,7 +205,6 @@ def plot_eigs_vs_frequency(eigs, freq, title='', xticks=None, yticks=None,
 		raise TypeError('`freq` must be ndarray type.')
 
 	# plot figure
-	dpi = 200
 	plt.figure(figsize=figsize, frameon=True, constrained_layout=False)
 	ax = plt.gca()
 	ratio = 1. / eigs.shape[1]
@@ -230,24 +220,14 @@ def plot_eigs_vs_frequency(eigs, freq, title='', xticks=None, yticks=None,
 	# axes management
 	plt.xlabel('Frequency')
 	plt.ylabel('Eigenvalues')
-	if xticks:
-		ax.set_xticks(xticks)
-		ax.set_xticklabels(xticks)
-	if yticks:
-		ax.set_yticks(yticks)
-		ax.set_yticklabels(yticks)
+	ax, xticks, yticks = _format_axes(ax, xticks, yticks)
 	if  equal_axes:
 		ax.set_aspect('equal')
 	if len(title) > 1:
 		plt.title(title)
 
 	# save or show plots
-	if filename:
-		if path == 'CWD': path = CWD
-		plt.savefig(os.path.join(path,filename), dpi=200)
-		plt.close()
-	else:
-		plt.show()
+	_save_show_plots(filename, path, plt)
 
 
 
@@ -292,12 +272,8 @@ def plot_eigs_vs_period(eigs, freq, title='', xticks=None, yticks=None,
 	ax.grid(True)
 
 	# set limits for axis
-	if xticks:
-		ax.set_xticks(xticks)
-		ax.set_xticklabels(xticks)
-	if yticks:
-		ax.set_yticks(yticks)
-		ax.set_yticklabels(yticks)
+	ax, xticks, yticks = _format_axes(ax, xticks, yticks)
+
 	if  equal_axes:
 		ax.set_aspect('equal')
 	plt.xlabel('Period')
@@ -307,19 +283,14 @@ def plot_eigs_vs_period(eigs, freq, title='', xticks=None, yticks=None,
 	ax.invert_xaxis()
 
 	# save or show plots
-	if filename:
-		if path == 'CWD': path = CWD
-		plt.savefig(os.path.join(path,filename), dpi=200)
-		plt.close()
-	else:
-		plt.show()
+	_save_show_plots(filename, path, plt)
 
 
 
 def plot_2D_modes_at_frequency(modes, freq_required, freq, vars_idx=[0], modes_idx=[0],
 	x1=None, x2=None, fftshift=False, imaginary=False, plot_max=False, coastlines='',
 	title='', xticks=None, yticks=None, figsize=(12,8), equal_axes=False, path='CWD',
-	filename=None):
+	filename=None,origin=None):
 	"""
 	Plot SPOD modes for 2D problems.
 
@@ -354,10 +325,8 @@ def plot_2D_modes_at_frequency(modes, freq_required, freq, vars_idx=[0], modes_i
 		Default is None.
 	"""
 	# get idx variables
-	if isinstance(vars_idx, int):
-		vars_idx = [vars_idx]
-	if not isinstance(vars_idx, (list,tuple)):
-		raise TypeError('`vars_idx` must be a list or tuple')
+	vars_idx = _check_vars(vars_idx)
+
 	# get idx modes
 	if isinstance(modes_idx, int):
 		modes_idx = [modes_idx]
@@ -373,10 +342,6 @@ def plot_2D_modes_at_frequency(modes, freq_required, freq, vars_idx=[0], modes_i
 		x1 = np.arange(modes.shape[0])
 		x2 = np.arange(modes.shape[1])
 
-	# split filename
-	if filename:
-		basename, ext = splitext(filename)
-
 	# loop over variables and modes
 	for var_id in vars_idx:
 
@@ -406,19 +371,19 @@ def plot_2D_modes_at_frequency(modes, freq_required, freq, vars_idx=[0], modes_i
 				real_ax = fig.add_subplot(1, 2, 1)
 				real = real_ax.contourf(
 					x1, x2, np.real(mode).T,
-					vmin=-np.abs(mode).max()*1.2,
-					vmax= np.abs(mode).max()*1.2)
+					vmin=-np.abs(mode).max()*1.,
+					vmax= np.abs(mode).max()*1.,
+					origin=origin)
 				imag_ax = fig.add_subplot(1, 2, 2)
 				imag = imag_ax.contourf(
 					x1, x2, np.imag(mode).T,
-					vmin=-np.abs(mode).max()*1.2,
-					vmax= np.abs(mode).max()*1.2)
+					vmin=-np.abs(mode).max()*1.,
+					vmax= np.abs(mode).max()*1.,
+					origin=origin)
 				if plot_max:
 					idx_x1,idx_x2 = np.where(np.abs(mode) == np.amax(np.abs(mode)))
-					real_ax.axhline(x1[idx_x1], xmin=0, xmax=1,color='k',linestyle='--')
-					real_ax.axvline(x2[idx_x2], ymin=0, ymax=1,color='k',linestyle='--')
-					imag_ax.axhline(x1[idx_x1], xmin=0, xmax=1,color='k',linestyle='--')
-					imag_ax.axvline(x2[idx_x2], ymin=0, ymax=1,color='k',linestyle='--')
+					real_ax = _apply_2d_vertical_lines(real_ax, x1, x2, idx_x1, idx_x2)
+					imag_ax  =_apply_2d_vertical_lines(imag_ax, x1, x2, idx_x1, idx_x2)
 				real_divider = make_axes_locatable(real_ax)
 				imag_divider = make_axes_locatable(imag_ax)
 				real_cax = real_divider.append_axes("right", size="5%", pad=0.05)
@@ -427,34 +392,14 @@ def plot_2D_modes_at_frequency(modes, freq_required, freq, vars_idx=[0], modes_i
 				plt.colorbar(imag, cax=imag_cax)
 
 				# overlay coastlines if required
-				if coastlines.lower() == 'regular':
-					coast = loadmat(os.path.join(CFD,'plotting_support','coast.mat'))
-					real_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
-					imag_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
-				elif coastlines.lower() == 'centred':
-					coast = loadmat(os.path.join(CFD,'plotting_support','coast_centred.mat'))
-					real_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
-					imag_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
+				real_ax = _apply_2d_coastlines(coastlines, real_ax)
+				imag_ax = _apply_2d_coastlines(coastlines, imag_ax)
 
 				# axis management
-				real_ax.set_xlim(np.nanmin(x1)*1.05,np.nanmax(x1)*1.05)
-				real_ax.set_ylim(np.nanmin(x2)*1.05,np.nanmax(x2)*1.05)
-				imag_ax.set_xlim(np.nanmin(x1)*1.05,np.nanmax(x1)*1.05)
-				imag_ax.set_ylim(np.nanmin(x2)*1.05,np.nanmax(x2)*1.05)
-				if xticks:
-					real_ax.set_xticks(xticks)
-					real_ax.set_xticklabels(xticks)
-					imag_ax.set_xticks(xticks)
-					imag_ax.set_xticklabels(xticks)
-				if yticks:
-					real_ax.set_yticks(yticks)
-					real_ax.set_yticklabels(yticks)
-					imag_ax.set_yticks(yticks)
-					imag_ax.set_yticklabels(yticks)
+				real_ax = _set_2d_axes_limits(real_ax, x1, x2)
+				imag_ax = _set_2d_axes_limits(imag_ax, x1, x2)
+				real_ax, xticks, yticks = _format_axes(real_ax, xticks, yticks)
+				imag_ax, xticks, yticks = _format_axes(imag_ax, xticks, yticks)
 				if equal_axes:
 					real_ax.set_aspect('equal')
 					imag_ax.set_aspect('equal')
@@ -469,36 +414,23 @@ def plot_2D_modes_at_frequency(modes, freq_required, freq, vars_idx=[0], modes_i
 				real_ax = plt.gca()
 				real = real_ax.contourf(
 					x1, x2, np.real(mode).T,
-					vmin=-np.abs(mode).max()*1.2,
-					vmax= np.abs(mode).max()*1.2)
+					vmin=-np.abs(mode).max()*1.,
+					vmax= np.abs(mode).max()*1.,
+					origin=origin)
 				if plot_max:
 					idx_x1,idx_x2 = np.where(np.abs(mode) == np.amax(np.abs(mode)))
-					real_ax.axhline(x1[idx_x1], xmin=0, xmax=1,color='k',linestyle='--')
-					real_ax.axvline(x2[idx_x2], ymin=0, ymax=1,color='k',linestyle='--')
+					real_ax = _apply_2d_vertical_lines(real_ax, x1, x2, idx_x1, idx_x2)
 				real_divider = make_axes_locatable(real_ax)
 				real_cax = real_divider.append_axes("right", size="5%", pad=0.05)
 				plt.colorbar(real, cax=real_cax)
+				real_ax = _apply_2d_coastlines(coastlines, real_ax)
 
-				# overlay coastlines if required
-				if coastlines.lower() == 'regular':
-					coast = loadmat(os.path.join(CFD,'plotting_support','coast.mat'))
-					real_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
-				elif coastlines.lower() == 'centred':
-					coast = loadmat(os.path.join(CFD,'plotting_support','coast_centred.mat'))
-					real_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
 				# axis management
 				if equal_axes:
 					real_ax.set_aspect('equal')
-				if xticks:
-					real_ax.set_xticks(xticks)
-					real_ax.set_xticklabels(xticks)
-				if yticks:
-					real_ax.set_yticks(yticks)
-					real_ax.set_yticklabels(yticks)
-				real_ax.set_xlim(np.nanmin(x1)*1.05,np.nanmax(x1)*1.05)
-				real_ax.set_ylim(np.nanmin(x2)*1.05,np.nanmax(x2)*1.05)
+
+				real_ax, xticks, yticks = _format_axes(real_ax, xticks, yticks)
+				real_ax = _set_2d_axes_limits(real_ax, x1, x2)
 				if len(title) > 1:
 					real_ax.set_title(title + \
 						', mode: {}, variable ID: {}'.format(mode_id, var_id))
@@ -511,6 +443,7 @@ def plot_2D_modes_at_frequency(modes, freq_required, freq, vars_idx=[0], modes_i
 			# save or show plots
 			if filename:
 				if path == 'CWD': path = CWD
+				basename, ext = splitext(filename)
 				filename = '{0}_var{1}_mode{2}{3}'.format(basename, var_id, mode_id, ext)
 				plt.savefig(os.path.join(path,filename),dpi=400)
 				plt.close(fig)
@@ -559,10 +492,8 @@ def plot_2D_mode_slice_vs_time(modes, freq_required, freq, vars_idx=[0],
 	"""
 
 	# get idx variables
-	if isinstance(vars_idx, int):
-		vars_idx = [vars_idx]
-	if not isinstance(vars_idx, (list,tuple)):
-		raise TypeError('`vars_idx` must be a list or tuple')
+	vars_idx = _check_vars(vars_idx)
+
 	# get idx modes
 	if isinstance(modes_idx, int):
 		modes_idx = [modes_idx]
@@ -578,10 +509,6 @@ def plot_2D_mode_slice_vs_time(modes, freq_required, freq, vars_idx=[0],
 		x1 = np.arange(modes.shape[0])
 		x2 = np.arange(modes.shape[1])
 
-	# split filename
-	if filename:
-		basename, ext = splitext(filename)
-
 	# calculate period and time vector
 	n_points = 50
 	period = 1. / freq_required
@@ -617,6 +544,7 @@ def plot_2D_mode_slice_vs_time(modes, freq_required, freq, vars_idx=[0],
 			tmp = np.fft.fftshift(tmp, axes=1)
 		idx_x1, idx_x2 = np.where(np.abs(tmp) == np.amax(np.abs(tmp)))
 
+		# loop over modes
 		for mode_id in modes_idx:
 
 			# select mode and fft-shift it
@@ -647,8 +575,7 @@ def plot_2D_mode_slice_vs_time(modes, freq_required, freq, vars_idx=[0],
 			ax.axhline(x2[idx_x2], xmin=0, xmax=1,color='k',linestyle='--')
 			ax.axvline(x1[idx_x1], ymin=0, ymax=1,color='k',linestyle='--')
 			# axis management
-			ax.set_xlim(np.nanmin(x1)*1.05,np.nanmax(x1)*1.05)
-			ax.set_ylim(np.nanmin(x2)*1.05,np.nanmax(x2)*1.05)
+			ax = _set_2d_axes_limits(ax, x1, x2)
 			ax_divider = make_axes_locatable(ax)
 			cax = ax_divider.append_axes("right", size="5%", pad=0.05)
 			plt.colorbar(ax_obj, cax=cax)
@@ -673,7 +600,6 @@ def plot_2D_mode_slice_vs_time(modes, freq_required, freq, vars_idx=[0],
 			# axis management
 			ax.set_xlim(np.nanmin(t )*1.05,np.nanmax(t )*1.05)
 			ax.set_ylim(np.nanmin(x2)*1.05,np.nanmax(x2)*1.05)
-			xlim = ax.get_xlim()
 			ax_divider = make_axes_locatable(ax)
 			cax = ax_divider.append_axes("bottom", size="5%", pad=0.65)
 			plt.colorbar(ax_obj, cax=cax, orientation="horizontal")
@@ -695,8 +621,7 @@ def plot_2D_mode_slice_vs_time(modes, freq_required, freq, vars_idx=[0],
 				vmin=np.nanmin(mode_phase_x1.real),
 				vmax=np.nanmax(mode_phase_x1.real))
 			# axis management
-			ax.set_xlim(np.nanmin(x1)*1.05,np.nanmax(x1)*1.05)
-			ax.set_ylim(np.nanmin(t )*1.05,np.nanmax(t )*1.05)
+			ax = _set_2d_axes_limits(ax, x1, x2)
 			ax_divider = make_axes_locatable(ax)
 			cax = ax_divider.append_axes("right", size="2.5%", pad=0.05)
 			plt.colorbar(ax_obj, cax=cax)
@@ -713,6 +638,7 @@ def plot_2D_mode_slice_vs_time(modes, freq_required, freq, vars_idx=[0],
 		# save or show plots
 		if filename:
 			if path == 'CWD': path = CWD
+			basename, ext = splitext(filename)
 			filename = '{0}_var{1}_mode{2}{3}'.format(basename, var_id, mode_id, ext)
 			plt.savefig(os.path.join(path,filename),dpi=400)
 			plt.close()
@@ -724,7 +650,7 @@ def plot_2D_mode_slice_vs_time(modes, freq_required, freq, vars_idx=[0],
 def plot_3D_modes_slice_at_frequency(modes, freq_required, freq, vars_idx=[0], modes_idx=[0],
 	x1=None, x2=None, x3=None, slice_dim=0, slice_id=None, fftshift=False, imaginary=False,
 	plot_max=False, coastlines='', title='', xticks=None, yticks=None, figsize=(12,8),
-	equal_axes=False, path='CWD', filename=None):
+	equal_axes=False, path='CWD', filename=None, origin=None):
 	"""
 	Plot SPOD modes for 3D problems.
 
@@ -764,10 +690,8 @@ def plot_3D_modes_slice_at_frequency(modes, freq_required, freq, vars_idx=[0], m
 	"""
 
 	# get idx variables
-	if isinstance(vars_idx, int):
-		vars_idx = [vars_idx]
-	if not isinstance(vars_idx, (list,tuple)):
-		raise TypeError('`vars_idx` must be a list or tuple')
+	vars_idx = _check_vars(vars_idx)
+
 	# get idx modes
 	if isinstance(modes_idx, int):
 		modes_idx = [modes_idx]
@@ -784,10 +708,6 @@ def plot_3D_modes_slice_at_frequency(modes, freq_required, freq, vars_idx=[0], m
 		x2 = np.arange(modes.shape[1])
 		x3 = np.arange(modes.shape[2])
 
-	# split filename
-	if filename:
-		basename, ext = splitext(filename)
-
 	# loop over variables and modes
 	for var_id in vars_idx:
 
@@ -800,35 +720,24 @@ def plot_3D_modes_slice_at_frequency(modes, freq_required, freq, vars_idx=[0], m
 			if mode_3d.ndim != 3:
 				raise ValueError('Dimension of the modes is not 3D.')
 
-			if slice_dim == 0:
-				if slice_id is None:
-					slice_id = np.argmax(mode_3d, axis=0)
-				mode = mode_3d[slice_id,:,:]
-				xx = x2
-				yy = x3
-				coastlines = ''
-			elif slice_dim == 1:
-				if slice_id is None:
-					slice_id = np.argmax(mode_3d, axis=1)
-				mode = mode_3d[:,slice_id,:]
-				xx = x1
-				yy = x3
-				coastlines = ''
-			elif slice_dim == 2:
-				if slice_id is None:
-					slice_id = np.argmax(mode_3d, axis=2)
-				mode = mode_3d[:,:,slice_id]
-				xx = x1
-				yy = x2
+			if slice_id is None: slice_id = 0
+			if slice_dim == 0: mode = mode_3d[slice_id,:,:]
+			elif slice_dim == 1: mode = mode_3d[:,slice_id,:]
+			elif slice_dim == 2: mode = mode_3d[:,:,slice_id]
+			# coord 1
+			if mode.shape[0] == x1.shape[0]: xx = x1; flag1 = 'x1'
+			elif mode.shape[0] == x2.shape[0]: xx = x2; flag1 = 'x2'
+			elif mode.shape[0] == x3.shape[0]: xx = x3; flag1 = 'x3'
+			# coord 2
+			if (mode.shape[1] == x1.shape[0]) and (flag1 != 'x1'): yy = x1; flag2 = 'x1'
+			elif (mode.shape[1] == x2.shape[0]) and (flag1 != 'x2'): yy = x2; flag2 = 'x2'
+			elif (mode.shape[1] == x3.shape[0]) and (flag1 != 'x3'): yy = x3; flag2 = 'x3'
+			coastlines = ''
 
 			# perform fft shift if required
 			if fftshift:
 				mode = np.fft.fftshift(mode, axes=1)
 
-			# check dimension axes and data
-			if xx.shape[0] != mode.shape[0] or yy.shape[0] != mode.shape[1]:
-				raise ValueError('Data dimension Z = (N,M); xx and yy must '
-								 'have dimension N and M, respectively.')
 			# plot data
 			if imaginary:
 
@@ -839,18 +748,18 @@ def plot_3D_modes_slice_at_frequency(modes, freq_required, freq, vars_idx=[0], m
 				real = real_ax.contourf(
 					xx, yy, np.real(mode).T,
 					vmin=-np.abs(mode).max(),
-					vmax= np.abs(mode).max())
+					vmax= np.abs(mode).max(),
+					origin=origin)
 				imag_ax = fig.add_subplot(1, 2, 2)
 				imag = imag_ax.contourf(
 					xx, yy, np.imag(mode).T,
 					vmin=-np.abs(mode).max(),
-					vmax= np.abs(mode).max())
+					vmax= np.abs(mode).max(),
+					origin=origin)
 				if plot_max:
 					idx_x1,idx_x2 = np.where(np.abs(mode) == np.amax(np.abs(mode)))
-					real_ax.axhline(x1[idx_x1], xmin=0, xmax=1,color='k',linestyle='--')
-					real_ax.axvline(x2[idx_x2], ymin=0, ymax=1,color='k',linestyle='--')
-					imag_ax.axhline(x1[idx_x1], xmin=0, xmax=1,color='k',linestyle='--')
-					imag_ax.axvline(x2[idx_x2], ymin=0, ymax=1,color='k',linestyle='--')
+					real_ax = _apply_2d_vertical_lines(real_ax, x1, x2, idx_x1, idx_x2)
+					imag_ax = _apply_2d_vertical_lines(imag_ax, x1, x2, idx_x1, idx_x2)
 				real_divider = make_axes_locatable(real_ax)
 				imag_divider = make_axes_locatable(imag_ax)
 				real_cax = real_divider.append_axes("right", size="5%", pad=0.05)
@@ -859,85 +768,60 @@ def plot_3D_modes_slice_at_frequency(modes, freq_required, freq, vars_idx=[0], m
 				plt.colorbar(imag, cax=imag_cax)
 
 				# overlay coastlines if required
-				if coastlines.lower() == 'regular':
-					coast = loadmat(os.path.join(CFD,'plotting_support','coast.mat'))
-					real_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
-					imag_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
-				elif coastlines.lower() == 'centred':
-					coast = loadmat(os.path.join(CFD,'plotting_support','coast_centred.mat'))
-					real_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
-					imag_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
+				real_ax = _apply_2d_coastlines(coastlines, real_ax)
+				imag_ax = _apply_2d_coastlines(coastlines, imag_ax)
 
 				# axis management
 				real_ax.set_xlim(np.nanmin(xx)*1.05,np.nanmax(xx)*1.05)
 				real_ax.set_ylim(np.nanmin(yy)*1.05,np.nanmax(yy)*1.05)
 				imag_ax.set_xlim(np.nanmin(xx)*1.05,np.nanmax(xx)*1.05)
 				imag_ax.set_ylim(np.nanmin(yy)*1.05,np.nanmax(yy)*1.05)
-				if xticks:
-					real_ax.set_xticks(xticks)
-					real_ax.set_xticklabels(xticks)
-					imag_ax.set_xticks(xticks)
-					imag_ax.set_xticklabels(xticks)
-				if yticks:
-					real_ax.set_yticks(yticks)
-					real_ax.set_yticklabels(yticks)
-					imag_ax.set_yticks(yticks)
-					imag_ax.set_yticklabels(yticks)
+				real_ax, xticks, yticks = _format_axes(real_ax, xticks, yticks)
+				imag_ax, xticks, yticks = _format_axes(imag_ax, xticks, yticks)
 				if equal_axes:
 					real_ax.set_aspect('equal')
 					imag_ax.set_aspect('equal')
+				real_ax.set_xlabel(flag1); imag_ax.set_xlabel(flag1)
+				real_ax.set_ylabel(flag2); imag_ax.set_ylabel(flag2)
 				if len(title) > 1:
-	 				fig.suptitle(title + \
+					fig.suptitle(title + \
 						', mode: {}, variable ID: {}'.format(mode_id, var_id))
 				else:
-	 				fig.suptitle('mode: {}, variable ID: {}'.format(mode_id, var_id))
+					fig.suptitle('mode: {}, variable ID: {}'.format(mode_id, var_id))
 				real_ax.set_title('Real part')
 				imag_ax.set_title('Imaginary part')
+
 			else:
 				fig = plt.figure(figsize=figsize)
 				real_ax = plt.gca()
 				real = real_ax.contourf(
 					xx, yy, np.real(mode).T,
 					vmin=-np.abs(mode).max(),
-					vmax= np.abs(mode).max())
+					vmax= np.abs(mode).max(),
+					origin=origin)
 				if plot_max:
 					idx_x1,idx_x2 = np.where(np.abs(mode) == np.amax(np.abs(mode)))
-					real_ax.axhline(x1[idx_x1], xmin=0, xmax=1,color='k',linestyle='--')
-					real_ax.axvline(x2[idx_x2], ymin=0, ymax=1,color='k',linestyle='--')
+					real_ax = _apply_2d_vertical_lines(real_ax, x1, x2, idx_x1, idx_x2)
 				real_divider = make_axes_locatable(real_ax)
 				real_cax = real_divider.append_axes("right", size="5%", pad=0.05)
 				plt.colorbar(real, cax=real_cax)
 
 				# overlay coastlines if required
-				if coastlines.lower() == 'regular':
-					coast = loadmat(os.path.join(CFD,'plotting_support','coast.mat'))
-					real_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
-				elif coastlines.lower() == 'centred':
-					coast = loadmat(os.path.join(CFD,'plotting_support','coast_centred.mat'))
-					real_ax.scatter(coast['coastlon'], coast['coastlat'],
-									marker='.', c='k', s=1)
+				real_ax = _apply_2d_coastlines(coastlines, real_ax)
 
 				# axis management
 				if equal_axes:
 					real_ax.set_aspect('equal')
-				if xticks:
-					real_ax.set_xticks(xticks)
-					real_ax.set_xticklabels(xticks)
-				if yticks:
-					real_ax.set_yticks(yticks)
-					real_ax.set_yticklabels(yticks)
+				real_ax, xticks, yticks = _format_axes(real_ax, xticks, yticks)
 				real_ax.set_xlim(np.nanmin(xx)*1.05,np.nanmax(xx)*1.05)
 				real_ax.set_ylim(np.nanmin(yy)*1.05,np.nanmax(yy)*1.05)
+				real_ax.set_xlabel(flag1)
+				real_ax.set_ylabel(flag2)
 				if len(title) > 1:
 					real_ax.set_title(title + \
-						', mode: {}, variable ID: {}'.format(mode_id, var_id))
+						', slice mode: {}, variable ID: {}'.format(mode_id, var_id))
 				else:
-					real_ax.set_title('mode: {}, variable ID: {}'.format(mode_id, var_id))
+					real_ax.set_title('slice mode: {}, variable ID: {}'.format(mode_id, var_id))
 
 			# padding between elements
 			plt.tight_layout(pad=2.)
@@ -945,6 +829,7 @@ def plot_3D_modes_slice_at_frequency(modes, freq_required, freq, vars_idx=[0], m
 			# save or show plots
 			if filename:
 				if path == 'CWD': path = CWD
+				basename, ext = splitext(filename)
 				filename = '{0}_var{1}_mode{2}{3}'.format(basename, var_id, mode_id, ext)
 				plt.savefig(os.path.join(path,filename),dpi=200)
 				plt.close(fig)
@@ -953,8 +838,6 @@ def plot_3D_modes_slice_at_frequency(modes, freq_required, freq, vars_idx=[0], m
 
 
 
-
-
 def plot_mode_tracers(modes, freq_required, freq, coords_list, x=None, vars_idx=[0],
 	modes_idx=[0], fftshift=False, title='', figsize=(12,8), path='CWD', filename=None):
 	"""
@@ -982,25 +865,19 @@ def plot_mode_tracers(modes, freq_required, freq, coords_list, x=None, vars_idx=
 	"""
 
 	# get idx variables
-	if isinstance(vars_idx, int):
-		vars_idx = [vars_idx]
-	if not isinstance(vars_idx, (list,tuple)):
-		raise TypeError('`vars_idx` must be a list or tuple')
+	vars_idx = _check_vars(vars_idx)
+
 	# get idx modes
 	if isinstance(modes_idx, int):
 		modes_idx = [modes_idx]
 	if not isinstance(modes_idx, (list,tuple)):
 		raise TypeError('`modes_idx` must be a list or tuple')
 
-	# split filename
-	if filename:
-		basename, ext = splitext(filename)
-
 	# if domain dimensions have not been passed as argument,
 	# use the data dimensions
 	if not coords_list:
 		print('You must provide coords to `plot_mode_tracers` '
-		      'in the form list(tuple(), tuple(), ...)')
+			  'in the form list(tuple(), tuple(), ...)')
 
 	# check the coord_list is indeed list
 	if not isinstance(coords_list, list):
@@ -1013,11 +890,10 @@ def plot_mode_tracers(modes, freq_required, freq, coords_list, x=None, vars_idx=
 
 	# get default coordinates if not provided
 	if x is None:
-		x = [np.arange(xdim[i]) for i in range(0,len(xdim))]
+		x = [np.arange(xdim[i]) for i in range(0, len(xdim))]
 
 	# get width and height figure
 	wsize = figsize[0]
-	hsize = figsize[1]
 
 	# calculate period and time vector
 	n_points = 100
@@ -1044,7 +920,7 @@ def plot_mode_tracers(modes, freq_required, freq, coords_list, x=None, vars_idx=
 					mode = np.fft.fftshift(mode, axes=1)
 				mode_point_phase = mode[idx_coords] * phase.conj()
 				ax = fig.add_subplot(spec[cnt,0])
-				ax_obj = ax.plot(t, np.real(mode_point_phase), 'k-')
+				_ = ax.plot(t, np.real(mode_point_phase), 'k-')
 				ax.set_ylabel('mode {}'.format(mode_id),
 								rotation=0,
 								labelpad=30,
@@ -1059,6 +935,7 @@ def plot_mode_tracers(modes, freq_required, freq, coords_list, x=None, vars_idx=
 			# save or show plots
 			if filename:
 				if path == 'CWD': path = CWD
+				basename, ext = splitext(filename)
 				filename = '{0}_coords{1}_var{2}_mode{3}{4}'.format(
 					basename, coords, var_id, mode_id, ext)
 				plt.savefig(os.path.join(path,filename), dpi=200)
@@ -1069,7 +946,7 @@ def plot_mode_tracers(modes, freq_required, freq, coords_list, x=None, vars_idx=
 
 
 def plot_2D_data(X, time_idx=[0], vars_idx=[0], x1=None, x2=None,
-	title='', coastlines='', figsize=(12,8), path='CWD', filename=None):
+	title='', coastlines='', figsize=(12,8), path='CWD', filename=None, origin=None):
 	"""
 	Plot 2D data.
 
@@ -1096,11 +973,10 @@ def plot_2D_data(X, time_idx=[0], vars_idx=[0], x1=None, x2=None,
 	# check dimensions
 	if X.ndim != 4:
 		raise ValueError('Dimension of data is not 2D.')
+
 	# get idx variables
-	if isinstance(vars_idx, int):
-		vars_idx = [vars_idx]
-	if not isinstance(vars_idx, (list,tuple)):
-		raise TypeError('`vars_idx` must be a list or tuple')
+	vars_idx = _check_vars(vars_idx)
+
 	# if domain dimensions have not been passed, use data dimensions
 	if x1 is None and x2 is None:
 		x1 = np.arange(X.shape[1])
@@ -1112,10 +988,6 @@ def plot_2D_data(X, time_idx=[0], vars_idx=[0], x1=None, x2=None,
 	if not isinstance(time_idx, (list,tuple)):
 		raise TypeError('`time_idx` must be a list or tuple')
 
-	# split filename
-	if filename:
-		basename, ext = splitext(filename)
-
 	# loop over variables and time indices
 	for var_id in vars_idx:
 		for time_id in time_idx:
@@ -1138,22 +1010,22 @@ def plot_2D_data(X, time_idx=[0], vars_idx=[0], x1=None, x2=None,
 			contour = plt.contourf(
 				x1, x2, x.T,
 				vmin=np.nanmin(x),
-				vmax=np.nanmax(x))
+				vmax=np.nanmax(x),
+				origin=origin)
 			fig.colorbar(contour)
 
 			# overlay coastlines if required
 			if coastlines.lower() == 'regular':
 				coast = loadmat(os.path.join(CFD,'plotting_support','coast.mat'))
-				plt.scatter(coast['coastlon'], coast['coastlat'],
-								marker='.', c='k', s=1)
+				plt.scatter(coast['coastlon'], coast['coastlat'], marker='.', c='k', s=1)
 			elif coastlines.lower() == 'centred':
 				coast = loadmat(os.path.join(CFD,'plotting_support','coast_centred.mat'))
-				plt.scatter(coast['coastlon'], coast['coastlat'],
-								marker='.', c='k', s=1)
+				plt.scatter(coast['coastlon'], coast['coastlat'], marker='.', c='k', s=1)
 
 			# save or show plots
 			if filename:
 				if path == 'CWD': path = CWD
+				basename, ext = splitext(filename)
 				filename = '{0}_var{1}_time{2}{3}'.format(basename, var_id, time_id, ext)
 				plt.savefig(os.path.join(path,filename), dpi=200)
 				plt.close(fig)
@@ -1186,22 +1058,16 @@ def plot_data_tracers(X, coords_list, x=None, time_limits=[0,10],
 	# check coord_list has correct shape and type
 	if not coords_list:
 		print('You must provide coords to `plot_mode_tracers` '
-		      'in the form list(tuple(), tuple(), ...)')
+			  'in the form list(tuple(), tuple(), ...)')
 	if not isinstance(coords_list, list):
 		raise TypeError('`coords` must be a list')
+
 	# get idx variables
-	if isinstance(vars_idx, int):
-		vars_idx = [vars_idx]
-	if not isinstance(vars_idx, (list,tuple)):
-		raise TypeError('`vars_idx` must be a list or tuple')
+	vars_idx = _check_vars(vars_idx)
 
 	# time range
 	time_range = list(range(time_limits[0],time_limits[-1]))
 
-	# split filename
-	if filename:
-		basename, ext = splitext(filename)
-
 	# get default coordinates if not provided
 	xdim = X[0,...,0].shape
 	if x is None:
@@ -1231,6 +1097,7 @@ def plot_data_tracers(X, coords_list, x=None, time_limits=[0,10],
 			# save or show plots
 			if filename:
 				if path == 'CWD': path = CWD
+				basename, ext = splitext(filename)
 				filename = '{0}_coords{1}_var{2}{3}'.format(basename, coords, var_id, ext)
 				plt.savefig(os.path.join(path,filename),dpi=400)
 				plt.close(fig)
@@ -1246,7 +1113,8 @@ def plot_data_tracers(X, coords_list, x=None, time_limits=[0,10],
 
 def generate_2D_data_video(X, time_limits=[0,10], vars_idx=None, sampling=1,
 	x1=None, x2=None, coastlines='', figsize=(12,8), path='CWD', filename='data_video.mp4'):
-	"""Make movie of 2D data.
+	"""
+		Make movie of 2D data.
 
 	:param numpy.ndarray X: 2D data to be plotted. \
 		First dimension must be time. Last dimension must be variable.
@@ -1269,11 +1137,10 @@ def generate_2D_data_video(X, time_limits=[0,10], vars_idx=None, sampling=1,
 	# check dimensions
 	if X.ndim != 4:
 		raise ValueError('Dimension of data is not 2D.')
+
 	# get idx variables
-	if isinstance(vars_idx, int):
-		vars_idx = [vars_idx]
-	if not isinstance(vars_idx, (list,tuple)):
-		raise TypeError('`vars_idx` must be a list or tuple')
+	vars_idx = _check_vars(vars_idx)
+
 	# if domain dimensions have not been passed, use data dimensions
 	if x1 is None and x2 is None:
 		x1 = np.arange(X.shape[1])
@@ -1283,13 +1150,6 @@ def generate_2D_data_video(X, time_limits=[0,10], vars_idx=None, sampling=1,
 	time_range = list(range(time_limits[0],time_limits[-1]))
 	time_range = time_range[0::sampling]
 
-	# filename
-	basename, ext = splitext(filename)
-
-	# figure dimensions
-	axes_ratio = (np.nanmax(x2) - np.nanmin(x2)) \
-			   / (np.nanmax(x1) - np.nanmin(x1))
-
 	# check dimension axes and data
 	if x1.shape[0] != X[0,:,:].shape[0] or \
 	   x2.shape[0] != X[0,:,:].shape[1]:
@@ -1297,10 +1157,6 @@ def generate_2D_data_video(X, time_limits=[0,10], vars_idx=None, sampling=1,
 			'Data dimension Z = (N,M); x1 and x2 '
 			'must have dimension N and M, respectively.')
 
-	# get figure size
-	wsize = figsize[0]
-	hsize = figsize[1]
-
 	# overlay coastlines if required
 	cst = False
 	if coastlines.lower() == 'regular':
@@ -1310,9 +1166,12 @@ def generate_2D_data_video(X, time_limits=[0,10], vars_idx=None, sampling=1,
 		coast = loadmat(os.path.join(CFD,'plotting_support','coast_centred.mat'))
 		cst = True
 
+	# filename
+	basename, ext = splitext(filename)
+
 	# Generate movie
-	vmin = np.nanmin(X)
-	vmax = np.nanmax(X)
+	#vmin = np.nanmin(X)
+	#vmax = np.nanmax(X)
 	vmean = np.nanmean(X)
 	for i in vars_idx:
 		fig = plt.figure()
@@ -1325,7 +1184,7 @@ def generate_2D_data_video(X, time_limits=[0,10], vars_idx=None, sampling=1,
 								vmin=-0.9*vmean,
 								vmax= 0.9*vmean),
 				 plt.scatter(coast['coastlon'],
-				 			 coast['coastlat'],
+							 coast['coastlat'],
 							 marker='.', c='k', s=1)]
 				for state in time_range
 			]
@@ -1347,3 +1206,65 @@ def generate_2D_data_video(X, time_limits=[0,10], vars_idx=None, sampling=1,
 		plt.close('all')
 
 # ---------------------------------------------------------------------------
+
+
+
+# Auxiliary plotting functions
+# ---------------------------------------------------------------------------
+
+def _format_axes(ax, xticks, yticks):
+	if xticks:
+		ax.set_xticks(xticks)
+		ax.set_xticklabels(xticks)
+	if yticks:
+		ax.set_yticks(yticks)
+		ax.set_yticklabels(yticks)
+	return ax, xticks, yticks
+
+
+
+def _check_vars(vars_idx):
+	if isinstance(vars_idx, int):
+	  vars_idx = [vars_idx]
+	if not isinstance(vars_idx, (list,tuple)):
+	    raise TypeError('`vars_idx` must be a list or tuple')
+	return vars_idx
+
+
+
+def _save_show_plots(filename, path, plt):
+	# save or show plots
+	if filename:
+	    if path == 'CWD': path = CWD
+	    plt.savefig(os.path.join(path,filename), dpi=200)
+	    plt.close()
+	else:
+	    plt.show()
+
+
+
+def _set_2d_axes_limits(ax, x1, x2):
+	ax.set_xlim(np.nanmin(x1)*1.05,np.nanmax(x1)*1.05)
+	ax.set_ylim(np.nanmin(x2)*1.05,np.nanmax(x2)*1.05)
+	return ax
+
+
+
+def _apply_2d_coastlines(coastlines, ax):
+	# overlay coastlines if required
+	if coastlines.lower() == 'regular':
+	    coast = loadmat(os.path.join(CFD, 'plotting_support','coast.mat'))
+	    ax.scatter(coast['coastlon'], coast['coastlat'], marker='.', c='k', s=1)
+	elif coastlines.lower() == 'centred':
+	    coast = loadmat(os.path.join(CFD,'plotting_support','coast_centred.mat'))
+	    ax.scatter(coast['coastlon'], coast['coastlat'], marker='.', c='k', s=1)
+	return ax
+
+
+
+def _apply_2d_vertical_lines(ax, x1, x2, idx1, idx2):
+	ax.axhline(x1[idx1], xmin=0, xmax=1,color='k',linestyle='--')
+	ax.axvline(x2[idx2], ymin=0, ymax=1,color='k',linestyle='--')
+	return ax
+
+# ---------------------------------------------------------------------------
